provider "vsphere" {
  vsphere_server = var.kaas_vsphere_server_ip
  user           = var.kaas_vsphere_capi_provider_username
  password       = var.kaas_vsphere_capi_provider_password

  # Enable this if your vSphere server has a self-signed certificate
  allow_unverified_ssl = true
}

data "vsphere_datacenter" "dc" {
  name = var.kaas_vsphere_datacenter_name
}

data "vsphere_resource_pool" "resource_pool" {
  name          = var.kaas_vsphere_resource_pool_path
  datacenter_id = data.vsphere_datacenter.dc.id
}

data "vsphere_datastore_cluster" "datastore_cluster" {
  name          = var.kaas_vsphere_datastorecluster_path
  datacenter_id = data.vsphere_datacenter.dc.id
}

data "vsphere_network" "network" {
  name          = var.kaas_vsphere_network_path
  datacenter_id = data.vsphere_datacenter.dc.id
}

data "vsphere_virtual_machine" "template_vm_linux" {
  name          = var.kaas_vsphere_os_template_path
  datacenter_id = data.vsphere_datacenter.dc.id
}

resource "tls_private_key" "ssh_key" {
  algorithm = "RSA"
  rsa_bits  = "4096"
}

resource "local_file" "ssh_public_key" {
  content  = tls_private_key.ssh_key.private_key_pem
  filename = "./ssh_keys/${var.cluster_name}.pem"
  provisioner "local-exec" {
    command = "chmod 0600 ${local_file.ssh_public_key.filename}"
  }
}

module "managers" {
  source               = "./modules/virtual_machine"
  quantity             = var.managers_count
  name_prefix          = "manager"
  resource_pool_id     = data.vsphere_resource_pool.resource_pool.id
  datastore_cluster_id = data.vsphere_datastore_cluster.datastore_cluster.id
  folder               = var.kaas_vsphere_machines_folder
  network_id           = data.vsphere_network.network.id
  template_vm          = data.vsphere_virtual_machine.template_vm_linux
  disk_size            = var.kaas_vsphere_machine_disk
  ram                  = var.kaas_vsphere_machine_ram
  num_cpu              = var.kaas_vsphere_numcpu
  guest_ssh_key_public = tls_private_key.ssh_key.public_key_openssh
  guest_ssh_user       = var.ssh_username
  cluster_name         = var.cluster_name
}

module "workers" {
  source               = "./modules/virtual_machine"
  quantity             = var.workers_count
  name_prefix          = "worker"
  resource_pool_id     = data.vsphere_resource_pool.resource_pool.id
  datastore_cluster_id = data.vsphere_datastore_cluster.datastore_cluster.id
  folder               = var.kaas_vsphere_machines_folder
  network_id           = data.vsphere_network.network.id
  template_vm          = data.vsphere_virtual_machine.template_vm_linux
  disk_size            = var.kaas_vsphere_machine_disk
  ram                  = var.kaas_vsphere_machine_ram
  num_cpu              = var.kaas_vsphere_numcpu
  guest_ssh_key_public = tls_private_key.ssh_key.public_key_openssh
  guest_ssh_user       = var.ssh_username
  cluster_name         = var.cluster_name
}

